@@ -18,7 +18,7 @@
  * @author     Adam Ashley <aashley@php.net>
  * @copyright  2001-2006 The PHP Group
  * @license    http://www.php.net/license/3_01.txt  PHP License 3.01
- * @version    CVS: $Id: DB.php,v 1.66 2006/09/21 02:00:26 aashley Exp $
+ * @version    CVS: $Id: DB.php,v 1.74 2008/04/04 07:57:02 aashley Exp $
  * @link       http://pear.php.net/package/Auth
  */
 
@@ -43,7 +43,7 @@ require_once 'DB.php';
  * @author     Adam Ashley <aashley@php.net>
  * @copyright  2001-2006 The PHP Group
  * @license    http://www.php.net/license/3_01.txt  PHP License 3.01
- * @version    Release: 1.4.0  File: $Revision: 1.66 $
+ * @version    Release: 1.6.1  File: $Revision: 1.74 $
  * @link       http://pear.php.net/package/Auth
  */
 class Auth_Container_DB extends Auth_Container
@@ -109,6 +109,8 @@ class Auth_Container_DB extends Auth_Container
      */
     function _connect($dsn)
     {
+        $this->log('Auth_Container_DB::_connect() called.', AUTH_LOG_DEBUG);
+
         if (is_string($dsn) || is_array($dsn)) {
             $this->db = DB::Connect($dsn, $this->options['db_options']);
         } elseif (is_subclass_of($dsn, 'db_common')) {
@@ -152,7 +154,14 @@ class Auth_Container_DB extends Auth_Container
             }
         }
         if ($this->options['auto_quote'] && $this->db->dsn['phptype'] != 'sqlite') {
-            $this->options['final_table'] = $this->db->quoteIdentifier($this->options['table']);
+            if (strpos('.', $this->options['table']) === false) {
+                $this->options['final_table'] = $this->db->quoteIdentifier($this->options['table']);
+            } else {
+                $t = explode('.', $this->options['table']);
+                for ($i = 0, $count = count($t); $i < $count; $i++)
+                    $t[$i] = $this->db->quoteIdentifier($t[$i]);
+                $this->options['final_table'] = implode('.', $t);
+            }
             $this->options['final_usernamecol'] = $this->db->quoteIdentifier($this->options['usernamecol']);
             $this->options['final_passwordcol'] = $this->db->quoteIdentifier($this->options['passwordcol']);
         } else {
@@ -205,6 +214,7 @@ class Auth_Container_DB extends Auth_Container
         $this->options['db_fields']   = '';
         $this->options['cryptType']   = 'md5';
         $this->options['db_options']  = array();
+        $this->options['db_where']    = '';
         $this->options['auto_quote']  = true;
     }
 
@@ -262,7 +272,7 @@ class Auth_Container_DB extends Auth_Container
 
         return '';
     }
-    
+
     // }}}
     // {{{ fetchData()
 
@@ -284,6 +294,7 @@ class Auth_Container_DB extends Auth_Container
      */
     function fetchData($username, $password, $isChallengeResponse=false)
     {
+        $this->log('Auth_Container_DB::fetchData() called.', AUTH_LOG_DEBUG);
         // Prepare for a database query
         $err = $this->_prepare();
         if ($err !== true) {
@@ -307,6 +318,14 @@ class Auth_Container_DB extends Auth_Container
                 " FROM ".$this->options['final_table'].
                 " WHERE ".$this->options['final_usernamecol']." = ".$this->db->quoteSmart($username);
 
+        // check if there is an optional parameter db_where
+        if ($this->options['db_where'] != '') {
+            // there is one, so add it to the query
+            $query .= " AND ".$this->options['db_where'];
+        }
+
+        $this->log('Running SQL against DB: '.$query, AUTH_LOG_DEBUG);
+
         $res = $this->db->getRow($query, null, DB_FETCHMODE_ASSOC);
 
         if (DB::isError($res)) {
@@ -326,12 +345,12 @@ class Auth_Container_DB extends Auth_Container
         if ($isChallengeResponse) {
             $res[$this->options['passwordcol']] = md5($res[$this->options['passwordcol']]
                     .$this->_auth_obj->session['loginchallenege']);
-            
+
             // UGLY cannot avoid without modifying verifyPassword
             if ($this->options['cryptType'] == 'md5') {
                 $res[$this->options['passwordcol']] = md5($res[$this->options['passwordcol']]);
             }
-            
+
             //print " Hashed Password [{$res[$this->options['passwordcol']]}]<br/>\n";
         }
 
@@ -344,8 +363,11 @@ class Auth_Container_DB extends Auth_Container
                     $key == $this->options['usernamecol']) {
                     continue;
                 }
+
+                $this->log('Storing additional field: '.$key, AUTH_LOG_DEBUG);
+
                 // Use reference to the auth object if exists
-                // This is because the auth session variable can change so a 
+                // This is because the auth session variable can change so a
                 // static call to setAuthData does not make sence
                 $this->_auth_obj->setAuthData($key, $value);
             }
@@ -366,6 +388,7 @@ class Auth_Container_DB extends Auth_Container
      */
     function listUsers()
     {
+        $this->log('Auth_Container_DB::listUsers() called.', AUTH_LOG_DEBUG);
         $err = $this->_prepare();
         if ($err !== true) {
             return PEAR::raiseError($err->getMessage(), $err->getCode());
@@ -390,6 +413,15 @@ class Auth_Container_DB extends Auth_Container
                          $sql_from,
                          $this->options['final_table']
                          );
+
+        // check if there is an optional parameter db_where
+        if ($this->options['db_where'] != '') {
+            // there is one, so add it to the query
+            $query .= " WHERE ".$this->options['db_where'];
+        }
+
+        $this->log('Running SQL against DB: '.$query, AUTH_LOG_DEBUG);
+
         $res = $this->db->getAll($query, null, DB_FETCHMODE_ASSOC);
 
         if (DB::isError($res)) {
@@ -400,6 +432,7 @@ class Auth_Container_DB extends Auth_Container
                 $retVal[] = $user;
             }
         }
+        $this->log('Found '.count($retVal).' users.', AUTH_LOG_DEBUG);
         return $retVal;
     }
 
@@ -418,15 +451,16 @@ class Auth_Container_DB extends Auth_Container
      */
     function addUser($username, $password, $additional = "")
     {
+        $this->log('Auth_Container_DB::addUser() called.', AUTH_LOG_DEBUG);
         $err = $this->_prepare();
         if ($err !== true) {
             return PEAR::raiseError($err->getMessage(), $err->getCode());
         }
 
-        if (   isset($this->options['cryptType']) 
+        if (   isset($this->options['cryptType'])
             && $this->options['cryptType'] == 'none') {
             $cryptFunction = 'strval';
-        } elseif (   isset($this->options['cryptType']) 
+        } elseif (   isset($this->options['cryptType'])
                   && function_exists($this->options['cryptType'])) {
             $cryptFunction = $this->options['cryptType'];
         } else {
@@ -459,6 +493,8 @@ class Auth_Container_DB extends Auth_Container
                          $additional_value
                          );
 
+        $this->log('Running SQL against DB: '.$query, AUTH_LOG_DEBUG);
+
         $res = $this->query($query);
 
         if (DB::isError($res)) {
@@ -481,17 +517,30 @@ class Auth_Container_DB extends Auth_Container
      */
     function removeUser($username)
     {
+        $this->log('Auth_Container_DB::removeUser() called.', AUTH_LOG_DEBUG);
+
         $err = $this->_prepare();
         if ($err !== true) {
             return PEAR::raiseError($err->getMessage(), $err->getCode());
         }
 
-        $query = sprintf("DELETE FROM %s WHERE %s = %s",
+        // check if there is an optional parameter db_where
+        if ($this->options['db_where'] != '') {
+            // there is one, so add it to the query
+            $where = " AND ".$this->options['db_where'];
+        } else {
+            $where = '';
+        }
+
+        $query = sprintf("DELETE FROM %s WHERE %s = %s %s",
                          $this->options['final_table'],
                          $this->options['final_usernamecol'],
-                         $this->db->quoteSmart($username)
+                         $this->db->quoteSmart($username),
+                         $where
                          );
 
+        $this->log('Running SQL against DB: '.$query, AUTH_LOG_DEBUG);
+
         $res = $this->query($query);
 
         if (DB::isError($res)) {
@@ -512,15 +561,16 @@ class Auth_Container_DB extends Auth_Container
      */
     function changePassword($username, $password)
     {
+        $this->log('Auth_Container_DB::changePassword() called.', AUTH_LOG_DEBUG);
         $err = $this->_prepare();
         if ($err !== true) {
             return PEAR::raiseError($err->getMessage(), $err->getCode());
         }
 
-        if (   isset($this->options['cryptType']) 
+        if (   isset($this->options['cryptType'])
             && $this->options['cryptType'] == 'none') {
             $cryptFunction = 'strval';
-        } elseif (   isset($this->options['cryptType']) 
+        } elseif (   isset($this->options['cryptType'])
                   && function_exists($this->options['cryptType'])) {
             $cryptFunction = $this->options['cryptType'];
         } else {
@@ -529,14 +579,25 @@ class Auth_Container_DB extends Auth_Container
 
         $password = $cryptFunction($password);
 
-        $query = sprintf("UPDATE %s SET %s = %s WHERE %s = %s",
+        // check if there is an optional parameter db_where
+        if ($this->options['db_where'] != '') {
+            // there is one, so add it to the query
+            $where = " AND ".$this->options['db_where'];
+        } else {
+            $where = '';
+        }
+
+        $query = sprintf("UPDATE %s SET %s = %s WHERE %s = %s %s",
                          $this->options['final_table'],
                          $this->options['final_passwordcol'],
                          $this->db->quoteSmart($password),
                          $this->options['final_usernamecol'],
-                         $this->db->quoteSmart($username)
+                         $this->db->quoteSmart($username),
+                         $where
                          );
 
+        $this->log('Running SQL against DB: '.$query, AUTH_LOG_DEBUG);
+
         $res = $this->query($query);
 
         if (DB::isError($res)) {
